/*=============================================================================
 * parser for CSP instances represented in XML format
 * 
 * Copyright (c) 2006 Olivier ROUSSEL (olivier.roussel <at> cril.univ-artois.fr)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *=============================================================================
 */
#ifndef _CSPInstanceInformationCallback_h_
#define _CSPInstanceInformationCallback_h_

/*
 * it is assumed that
 * - domains definitions are well formed (no duplicates)
 */

#include <set>
#include <map>

#include "ExpressionParser.h"
#include "CSPParserCallback.h"

namespace CSPXMLParser
{
  using namespace std;

// definition of the functions which are called by the parser when it
// reads new information in the file. These functions are in charge of
// creating the data structure that the solver needs to do his job
  class CSPInstanceInformationCallback : public CSPParserCallback
  {
  public:
    unsigned int nbVar,nbConstr,maxArity,maxDomSize;
    unsigned int nbPredicateConstr,nbRelationConstr;
    map<string,int> globalConstraints; // name and number of
				       // occurrences of a given
				       // global constraint
  private:
    unsigned int curDomSize;

    string currentConstraintName,currentConstraintReference;
    CSPDefinitionType currentConstraintType;

  public:
    CSPInstanceInformationCallback()
    {
      nbVar=0;
      nbConstr=0;
      maxArity=0;
      maxDomSize=0;

      nbPredicateConstr=0;
      nbRelationConstr=0;
    }

    virtual void beginInstance(const string & name)
    {
    }

    virtual void beginDomainsSection(int nbDomains) 
    {  
    }
  
    virtual void beginDomain(const string & name, int idDomain, int nbValue) 
    {
      curDomSize=0;
    }

    void addDomainValue(int v) 
    {
      curDomSize++;
    }

    virtual void addDomainValue(int first,int last) 
    {
      curDomSize+=last-first+1;
    }

    virtual void endDomain() 
    {
      maxDomSize=std::max(maxDomSize,curDomSize);
    }

    /**
     * end the definition of all domains
     */
    virtual void endDomainsSection() 
    {
    }

    virtual void beginVariablesSection(int nbVariables) 
    {
      nbVar=0;
    }
  
    virtual void addVariable(const string & name, int idVar,
			     const string & domain, int idDomain) 
    {
      ++nbVar;
    }

    virtual void endVariablesSection() 
    {
    }

    virtual void beginRelationsSection(int nbRelations) 
    {
    }

  
    virtual void beginRelation(const string & name, int idRel,
			       int arity, int nbTuples, bool isSupport) 
    {
    }

    virtual void addRelationTuple(int arity, int tuple[]) 
    {
    }

    virtual void endRelation() 
    {
    }

    virtual void endRelationsSection() 
    {
    }

    virtual void beginPredicatesSection(int nbPredicates) 
    {
    }
  
    virtual void beginPredicate(const string & name, int idPred) 
    {
    }

    virtual void addFormalParameter(int pos, const string & name, 
				    const string & type) 
    {
    }

    virtual void predicateExpression(AST *tree) 
    {
    }

    virtual void predicateExpression(const string &expr) 
    {
    }

    virtual void endPredicate() 
    {
    }

    virtual void endPredicatesSection() 
    {
    }

    virtual void beginConstraintsSection(int nbConstraints) 
    {
    }
  
    virtual void beginConstraint(const string & name, int idConstr,
				 int arity, 
				 const string & reference, 
				 CSPDefinitionType type, int id)
    {
      nbConstr++;
      maxArity=std::max(maxArity,(unsigned int)arity);

      switch(type)
      {
      case RelationType:
	nbRelationConstr++;
	break;

      case PredicateType:
	nbPredicateConstr++;
	break;

      case GlobalConstraintType:
	// get rid of "global:"
	if (reference.substr(0,7)=="global:")
	  globalConstraints[reference.substr(7)]++;
	else
	  globalConstraints[reference]++;
	break;
      }
    }

    virtual void addVariableToConstraint(const string & name, int idVar) 
    {
    }

    virtual void addEffectiveParameter(int pos, const string & expr) 
    {
    }

    virtual void addEffectiveParameter(int pos, AST *tree) 
    {
    }

    virtual void addEffectiveParameterVariable(int pos, 
					       const string & name, 
					       int idVar) 
    {
    }

    virtual void addEffectiveParameterInteger(int pos, 
					    int value)
    {
    }

    virtual void addEffectiveParameterList(int pos) 
    {
    }

    /**
     * indicate the start of a list
     */
    virtual void beginList() 
    {
    }

    virtual void addListVariable(int pos, const string & name, int idVar) 
    {
    }

    virtual void addListInteger(int pos, int value) 
    {
    }

    virtual void endList() 
    {
    }

    virtual void endConstraint() 
    {
    }

    /**
     * end the definition of all constraints
     */
    virtual void endConstraintsSection() 
    {
    }

    /**
     * signal the end of parsing
     */
    virtual void endInstance() 
    {
    }

  };

} // namespace

// Local Variables:
// mode: C++
// End:

#endif

